# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import (
    KnowledgeBaseDataSource,
)
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.shared import GenericDeleteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKnowledgeBaseDataSources:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
            },
            name="name",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
                "s3_prefix": "s3_prefix",
            },
            name="name",
            data_source_auth_config={
                "client_secret": "client_secret",
                "source": "SharePoint",
                "encrypted": True,
            },
            description="description",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.knowledge_base_data_sources.with_raw_response.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
            },
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = response.parse()
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.knowledge_base_data_sources.with_streaming_response.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
            },
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = response.parse()
            assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.retrieve(
            "knowledge_base_data_source_id",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.knowledge_base_data_sources.with_raw_response.retrieve(
            "knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = response.parse()
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.knowledge_base_data_sources.with_streaming_response.retrieve(
            "knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = response.parse()
            assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            client.knowledge_base_data_sources.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
            data_source_auth_config={
                "client_secret": "client_secret",
                "source": "SharePoint",
                "encrypted": True,
            },
            description="description",
            name="name",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.knowledge_base_data_sources.with_raw_response.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = response.parse()
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.knowledge_base_data_sources.with_streaming_response.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = response.parse()
            assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            client.knowledge_base_data_sources.with_raw_response.update(
                knowledge_base_data_source_id="",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.list()
        assert_matches_type(SyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.list(
            account_id="account_id",
            limit=1,
            page=1,
        )
        assert_matches_type(SyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.knowledge_base_data_sources.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = response.parse()
        assert_matches_type(SyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.knowledge_base_data_sources.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = response.parse()
            assert_matches_type(
                SyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.delete(
            "knowledge_base_data_source_id",
        )
        assert_matches_type(GenericDeleteResponse, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.knowledge_base_data_sources.with_raw_response.delete(
            "knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = response.parse()
        assert_matches_type(GenericDeleteResponse, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.knowledge_base_data_sources.with_streaming_response.delete(
            "knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = response.parse()
            assert_matches_type(GenericDeleteResponse, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            client.knowledge_base_data_sources.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_verify(self, client: SGPClient) -> None:
        knowledge_base_data_source = client.knowledge_base_data_sources.verify(
            "knowledge_base_data_source_id",
        )
        assert_matches_type(object, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_raw_response_verify(self, client: SGPClient) -> None:
        response = client.knowledge_base_data_sources.with_raw_response.verify(
            "knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = response.parse()
        assert_matches_type(object, knowledge_base_data_source, path=["response"])

    @parametrize
    def test_streaming_response_verify(self, client: SGPClient) -> None:
        with client.knowledge_base_data_sources.with_streaming_response.verify(
            "knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = response.parse()
            assert_matches_type(object, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_verify(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            client.knowledge_base_data_sources.with_raw_response.verify(
                "",
            )


class TestAsyncKnowledgeBaseDataSources:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
            },
            name="name",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
                "s3_prefix": "s3_prefix",
            },
            name="name",
            data_source_auth_config={
                "client_secret": "client_secret",
                "source": "SharePoint",
                "encrypted": True,
            },
            description="description",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_base_data_sources.with_raw_response.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
            },
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = await response.parse()
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_base_data_sources.with_streaming_response.create(
            account_id="account_id",
            data_source_config={
                "aws_account_id": "aws_account_id",
                "aws_region": "aws_region",
                "s3_bucket": "s3_bucket",
                "source": "S3",
            },
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = await response.parse()
            assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.retrieve(
            "knowledge_base_data_source_id",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_base_data_sources.with_raw_response.retrieve(
            "knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = await response.parse()
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_base_data_sources.with_streaming_response.retrieve(
            "knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = await response.parse()
            assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            await async_client.knowledge_base_data_sources.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
            data_source_auth_config={
                "client_secret": "client_secret",
                "source": "SharePoint",
                "encrypted": True,
            },
            description="description",
            name="name",
        )
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_base_data_sources.with_raw_response.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = await response.parse()
        assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_base_data_sources.with_streaming_response.update(
            knowledge_base_data_source_id="knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = await response.parse()
            assert_matches_type(KnowledgeBaseDataSource, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            await async_client.knowledge_base_data_sources.with_raw_response.update(
                knowledge_base_data_source_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.list()
        assert_matches_type(AsyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.list(
            account_id="account_id",
            limit=1,
            page=1,
        )
        assert_matches_type(AsyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_base_data_sources.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = await response.parse()
        assert_matches_type(AsyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_base_data_sources.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = await response.parse()
            assert_matches_type(
                AsyncPageResponse[KnowledgeBaseDataSource], knowledge_base_data_source, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.delete(
            "knowledge_base_data_source_id",
        )
        assert_matches_type(GenericDeleteResponse, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_base_data_sources.with_raw_response.delete(
            "knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = await response.parse()
        assert_matches_type(GenericDeleteResponse, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_base_data_sources.with_streaming_response.delete(
            "knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = await response.parse()
            assert_matches_type(GenericDeleteResponse, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            await async_client.knowledge_base_data_sources.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_verify(self, async_client: AsyncSGPClient) -> None:
        knowledge_base_data_source = await async_client.knowledge_base_data_sources.verify(
            "knowledge_base_data_source_id",
        )
        assert_matches_type(object, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_raw_response_verify(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.knowledge_base_data_sources.with_raw_response.verify(
            "knowledge_base_data_source_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base_data_source = await response.parse()
        assert_matches_type(object, knowledge_base_data_source, path=["response"])

    @parametrize
    async def test_streaming_response_verify(self, async_client: AsyncSGPClient) -> None:
        async with async_client.knowledge_base_data_sources.with_streaming_response.verify(
            "knowledge_base_data_source_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base_data_source = await response.parse()
            assert_matches_type(object, knowledge_base_data_source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_verify(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `knowledge_base_data_source_id` but received ''"
        ):
            await async_client.knowledge_base_data_sources.with_raw_response.verify(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["KnowledgeBaseQueryParams", "WildcardFilters"]


class KnowledgeBaseQueryParams(TypedDict, total=False):
    query: Required[str]
    """
    The natural language query to be answered by referencing the data ingested into
    the knowledge base
    """

    top_k: Required[int]
    """Number of chunks to return.

    Must be greater than 0 if specified. If not specified, all chunks will be
    returned.
    """

    include_embeddings: bool
    """Whether or not to include the embeddings for each chunk"""

    metadata_filters: object
    """Optional filter by metadata fields, encoded as a JSON object"""

    verbose: bool
    """Enable or disable verbose logging"""

    wildcard_filters: Dict[str, WildcardFilters]
    """Optional wildcard filter for fields.

    Only fields that are part of the metadata will be accepted.
    """


class WildcardFilters(TypedDict, total=False):
    value: Required[str]
    """Wildcard filter string"""

    case_insensitive: bool
    """Check case on wildcard string"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from ..shared.chunk_extra_info_schema import ChunkExtraInfoSchema
from ..shared.result_schema_generation import ResultSchemaGeneration
from ..shared.string_extra_info_schema import StringExtraInfoSchema
from ..evaluation_datasets.flexible_chunk import FlexibleChunk
from ..evaluation_datasets.flexible_message import FlexibleMessage

__all__ = [
    "TestCaseResult",
    "GenerationTestCaseResultResponse",
    "FlexibleTestCaseResultResponse",
    "FlexibleTestCaseResultResponseTestCaseEvaluationData",
    "FlexibleTestCaseResultResponseTestCaseEvaluationDataGenerationExtraInfo",
]


class GenerationTestCaseResultResponse(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str

    evaluation_dataset_version_num: str

    evaluation_id: str

    label_status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    test_case_evaluation_data: ResultSchemaGeneration

    test_case_id: str

    annotated_by_user_id: Optional[str] = None
    """The user who annotated the task."""

    application_test_case_output_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    audit_comment: Optional[str] = None

    audit_required: Optional[bool] = None

    audit_status: Optional[Literal["UNAUDITED", "FIXED", "APPROVED"]] = None
    """An enumeration."""

    completed_at: Optional[datetime] = None

    result: Optional[object] = None

    test_case_evaluation_data_schema: Optional[Literal["GENERATION"]] = None

    time_spent_labeling_s: Optional[int] = None
    """The time spent labeling in seconds."""


FlexibleTestCaseResultResponseTestCaseEvaluationDataGenerationExtraInfo: TypeAlias = Annotated[
    Union[ChunkExtraInfoSchema, StringExtraInfoSchema], PropertyInfo(discriminator="schema_type")
]


class FlexibleTestCaseResultResponseTestCaseEvaluationData(BaseModel):
    generation_output: Union[
        str,
        float,
        List[FlexibleChunk],
        List[FlexibleMessage],
        List[object],
        Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]],
        object,
    ]

    generation_extra_info: Optional[FlexibleTestCaseResultResponseTestCaseEvaluationDataGenerationExtraInfo] = None


class FlexibleTestCaseResultResponse(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str

    evaluation_dataset_version_num: str

    evaluation_id: str

    label_status: Literal["PENDING", "COMPLETED", "FAILED"]
    """An enumeration."""

    test_case_evaluation_data: FlexibleTestCaseResultResponseTestCaseEvaluationData

    test_case_id: str

    annotated_by_user_id: Optional[str] = None
    """The user who annotated the task."""

    application_test_case_output_id: Optional[str] = None

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    audit_comment: Optional[str] = None

    audit_required: Optional[bool] = None

    audit_status: Optional[Literal["UNAUDITED", "FIXED", "APPROVED"]] = None
    """An enumeration."""

    completed_at: Optional[datetime] = None

    result: Optional[object] = None

    test_case_evaluation_data_schema: Optional[Literal["FLEXIBLE"]] = None

    time_spent_labeling_s: Optional[int] = None
    """The time spent labeling in seconds."""


TestCaseResult: TypeAlias = Annotated[
    Union[GenerationTestCaseResultResponse, FlexibleTestCaseResultResponse],
    PropertyInfo(discriminator="test_case_evaluation_data_schema"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "KnowledgeBaseCreateParams",
    "EmbeddingConfig",
    "EmbeddingConfigEmbeddingConfigModelsAPI",
    "EmbeddingConfigEmbeddingConfigBase",
]


class KnowledgeBaseCreateParams(TypedDict, total=False):
    embedding_config: Required[EmbeddingConfig]
    """The configuration of the embedding"""

    knowledge_base_name: Required[str]
    """A unique name for the knowledge base"""

    account_id: str
    """Account to create knowledge base in.

    If you have access to more than one account, you must specify an account_id
    """

    metadata: object
    """Metadata associated with the knowledge base"""


class EmbeddingConfigEmbeddingConfigModelsAPI(TypedDict, total=False):
    model_deployment_id: Required[str]
    """The ID of the deployment of the created model in the Models API V3."""

    type: Required[Literal["models_api"]]
    """The type of the embedding configuration."""


class EmbeddingConfigEmbeddingConfigBase(TypedDict, total=False):
    embedding_model: Required[
        Literal[
            "sentence-transformers/all-MiniLM-L12-v2",
            "sentence-transformers/all-mpnet-base-v2",
            "sentence-transformers/multi-qa-distilbert-cos-v1",
            "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
            "openai/text-embedding-ada-002",
            "openai/text-embedding-3-small",
            "openai/text-embedding-3-large",
            "embed-english-v3.0",
            "embed-english-light-v3.0",
            "embed-multilingual-v3.0",
        ]
    ]
    """An enumeration."""

    type: Literal["base"]
    """The type of the embedding configuration."""


EmbeddingConfig: TypeAlias = Union[EmbeddingConfigEmbeddingConfigModelsAPI, EmbeddingConfigEmbeddingConfigBase]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List
from typing_extensions import TypedDict

__all__ = ["GenerationJobCreateParams"]


class GenerationJobCreateParams(TypedDict, total=False):
    advanced_config: Dict[str, List[str]]
    """Advanced configuration for the evaluation dataset generation job."""

    custom_instructions: str
    """Custom instructions for test case generation"""

    group_by_artifact_id: bool
    """
    If this flag is true, for every generated test case, the chunks used to generate
    it will be guaranteed to be from the same document (artifact).
    """

    harms_list: List[str]
    """List of harms to be used for the evaluation dataset generation.

    If not provided, generation will use the knowledge base id.
    """

    num_test_cases: int
    """Number of test cases to generate for the evaluation dataset"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ChunkListParams"]


class ChunkListParams(TypedDict, total=False):
    chunk_id: str
    """Optional search by chunk_id"""

    max_chunks: int
    """Maximum number of chunks returned by the get_chunks endpoint.

    Defaults to 10 and cannot be greater than 2000.
    """

    metadata_filters: str
    """Optional search by metadata fields, encoded as a JSON object"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable
from typing_extensions import Literal, Required, TypedDict

from .application_edge_param import ApplicationEdgeParam
from .application_node_param import ApplicationNodeParam

__all__ = ["ApplicationValidateParams", "Overrides"]


class ApplicationValidateParams(TypedDict, total=False):
    edges: Required[Iterable[ApplicationEdgeParam]]
    """List of edges in the application graph"""

    nodes: Required[Iterable[ApplicationNodeParam]]
    """List of nodes in the application graph"""

    version: Required[Literal["V0"]]
    """Version of the application schema"""

    overrides: Dict[str, Overrides]
    """Optional overrides for the application"""


class Overrides(TypedDict, total=False):
    artifact_ids_filter: List[str]

    artifact_name_regex: List[str]

    type: Literal["knowledge_base_schema"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["ChunkSynthesisParams", "Chunk"]


class ChunkSynthesisParams(TypedDict, total=False):
    chunks: Required[Iterable[Chunk]]
    """List of chunks to use to synthesize the response."""

    query: Required[str]
    """Natural language query to resolve using the supplied chunks."""


class Chunk(TypedDict, total=False):
    chunk_id: Required[str]
    """The unique ID of the chunk with embedding"""

    score: Required[float]
    """
    A number between 0 and 1 representing how similar a chunk's embedding is to the
    query embedding. Higher numbers mean that this chunk with embedding is more
    similar.
    """

    text: Required[str]
    """The text associated with the chunk"""

    attachment_url: str
    """Original attachment URL from which this chunk got its data from"""

    embedding: Iterable[float]
    """The vector embedding of the text associated with the chunk"""

    metadata: object
    """
    Any additional key value pairs of information stored by you on the chunk with
    embedding
    """

    title: str
    """Title for this chunk, for example the file name"""

    user_supplied_metadata: object
    """
    Any additional key value pairs of information returned from the custom chunking.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.evaluation_datasets import (
    autogenerated_draft_test_case_list_params,
    autogenerated_draft_test_case_create_params,
    autogenerated_draft_test_case_update_params,
    autogenerated_draft_test_case_approve_params,
)
from ...types.shared.generic_delete_response import GenericDeleteResponse
from ...types.evaluation_datasets.auto_generated_draft_test_case import AutoGeneratedDraftTestCase
from ...types.evaluation_datasets.autogenerated_draft_test_case_list_response import (
    AutogeneratedDraftTestCaseListResponse,
)
from ...types.evaluation_datasets.approve_auto_generated_draft_test_case_response import (
    ApproveAutoGeneratedDraftTestCaseResponse,
)

__all__ = ["AutogeneratedDraftTestCasesResource", "AsyncAutogeneratedDraftTestCasesResource"]


class AutogeneratedDraftTestCasesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AutogeneratedDraftTestCasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AutogeneratedDraftTestCasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AutogeneratedDraftTestCasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AutogeneratedDraftTestCasesResourceWithStreamingResponse(self)

    def create(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str,
        approved: bool,
        test_case_data: autogenerated_draft_test_case_create_params.TestCaseData,
        schema_type: Literal["GENERATION"] | NotGiven = NOT_GIVEN,
        topic_str: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCase:
        """
        ### Description

        Creates a autogenerated draft test case

        ### Details

        This API can be used to create a autogenerated draft test case. To use this API,
        review the request schema and pass in all fields that are required to create a
        autogenerated draft test case.

        Args:
          account_id: The ID of the account that owns the given entity.

          approved: Boolean to track whether or not the draft test case is approved

          test_case_data: The data for the test case in a format matching the provided schema_type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "approved": approved,
                    "test_case_data": test_case_data,
                    "schema_type": schema_type,
                    "topic_str": topic_str,
                },
                autogenerated_draft_test_case_create_params.AutogeneratedDraftTestCaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoGeneratedDraftTestCase,
        )

    def retrieve(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCase:
        """
        ### Description

        Get the test case by its id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoGeneratedDraftTestCase,
        )

    def update(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        approved: bool | NotGiven = NOT_GIVEN,
        schema_type: Literal["GENERATION"] | NotGiven = NOT_GIVEN,
        test_case_data: autogenerated_draft_test_case_update_params.TestCaseData | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCase:
        """
        ### Description

        Update a test case draft.

        Args:
          approved: Boolean to track whether or not the draft test case is approved

          test_case_data: The data for the test case in a format matching the provided schema_type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return self._patch(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}",
            body=maybe_transform(
                {
                    "approved": approved,
                    "schema_type": schema_type,
                    "test_case_data": test_case_data,
                },
                autogenerated_draft_test_case_update_params.AutogeneratedDraftTestCaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoGeneratedDraftTestCase,
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        check_missing_chunks: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[AutogeneratedDraftTestCaseListResponse]:
        """
        List Autogenerated Draft Test Cases

        Args:
          check_missing_chunks: Check if test cases are based on missing chunks

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases",
            page=SyncPageResponse[AutogeneratedDraftTestCaseListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "check_missing_chunks": check_missing_chunks,
                        "limit": limit,
                        "page": page,
                    },
                    autogenerated_draft_test_case_list_params.AutogeneratedDraftTestCaseListParams,
                ),
            ),
            model=AutogeneratedDraftTestCaseListResponse,
        )

    def delete(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """### Description

        Delete a specified draft test case.

        This version will not be retained.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return self._delete(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    def approve(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        force: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApproveAutoGeneratedDraftTestCaseResponse:
        """
        Approve Autogenerated Draft Test Case

        Args:
          force: Force approve a failing test case

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}/approve",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"force": force},
                    autogenerated_draft_test_case_approve_params.AutogeneratedDraftTestCaseApproveParams,
                ),
            ),
            cast_to=ApproveAutoGeneratedDraftTestCaseResponse,
        )


class AsyncAutogeneratedDraftTestCasesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAutogeneratedDraftTestCasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAutogeneratedDraftTestCasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse(self)

    async def create(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str,
        approved: bool,
        test_case_data: autogenerated_draft_test_case_create_params.TestCaseData,
        schema_type: Literal["GENERATION"] | NotGiven = NOT_GIVEN,
        topic_str: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCase:
        """
        ### Description

        Creates a autogenerated draft test case

        ### Details

        This API can be used to create a autogenerated draft test case. To use this API,
        review the request schema and pass in all fields that are required to create a
        autogenerated draft test case.

        Args:
          account_id: The ID of the account that owns the given entity.

          approved: Boolean to track whether or not the draft test case is approved

          test_case_data: The data for the test case in a format matching the provided schema_type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "approved": approved,
                    "test_case_data": test_case_data,
                    "schema_type": schema_type,
                    "topic_str": topic_str,
                },
                autogenerated_draft_test_case_create_params.AutogeneratedDraftTestCaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoGeneratedDraftTestCase,
        )

    async def retrieve(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCase:
        """
        ### Description

        Get the test case by its id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return await self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoGeneratedDraftTestCase,
        )

    async def update(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        approved: bool | NotGiven = NOT_GIVEN,
        schema_type: Literal["GENERATION"] | NotGiven = NOT_GIVEN,
        test_case_data: autogenerated_draft_test_case_update_params.TestCaseData | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCase:
        """
        ### Description

        Update a test case draft.

        Args:
          approved: Boolean to track whether or not the draft test case is approved

          test_case_data: The data for the test case in a format matching the provided schema_type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return await self._patch(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}",
            body=await async_maybe_transform(
                {
                    "approved": approved,
                    "schema_type": schema_type,
                    "test_case_data": test_case_data,
                },
                autogenerated_draft_test_case_update_params.AutogeneratedDraftTestCaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoGeneratedDraftTestCase,
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        check_missing_chunks: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[
        AutogeneratedDraftTestCaseListResponse, AsyncPageResponse[AutogeneratedDraftTestCaseListResponse]
    ]:
        """
        List Autogenerated Draft Test Cases

        Args:
          check_missing_chunks: Check if test cases are based on missing chunks

          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases",
            page=AsyncPageResponse[AutogeneratedDraftTestCaseListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "check_missing_chunks": check_missing_chunks,
                        "limit": limit,
                        "page": page,
                    },
                    autogenerated_draft_test_case_list_params.AutogeneratedDraftTestCaseListParams,
                ),
            ),
            model=AutogeneratedDraftTestCaseListResponse,
        )

    async def delete(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """### Description

        Delete a specified draft test case.

        This version will not be retained.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return await self._delete(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    async def approve(
        self,
        autogenerated_draft_test_case_id: str,
        *,
        evaluation_dataset_id: str,
        force: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApproveAutoGeneratedDraftTestCaseResponse:
        """
        Approve Autogenerated Draft Test Case

        Args:
          force: Force approve a failing test case

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not autogenerated_draft_test_case_id:
            raise ValueError(
                f"Expected a non-empty value for `autogenerated_draft_test_case_id` but received {autogenerated_draft_test_case_id!r}"
            )
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/autogenerated-draft-test-cases/{autogenerated_draft_test_case_id}/approve",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"force": force},
                    autogenerated_draft_test_case_approve_params.AutogeneratedDraftTestCaseApproveParams,
                ),
            ),
            cast_to=ApproveAutoGeneratedDraftTestCaseResponse,
        )


class AutogeneratedDraftTestCasesResourceWithRawResponse:
    def __init__(self, autogenerated_draft_test_cases: AutogeneratedDraftTestCasesResource) -> None:
        self._autogenerated_draft_test_cases = autogenerated_draft_test_cases

        self.create = to_raw_response_wrapper(
            autogenerated_draft_test_cases.create,
        )
        self.retrieve = to_raw_response_wrapper(
            autogenerated_draft_test_cases.retrieve,
        )
        self.update = to_raw_response_wrapper(
            autogenerated_draft_test_cases.update,
        )
        self.list = to_raw_response_wrapper(
            autogenerated_draft_test_cases.list,
        )
        self.delete = to_raw_response_wrapper(
            autogenerated_draft_test_cases.delete,
        )
        self.approve = to_raw_response_wrapper(
            autogenerated_draft_test_cases.approve,
        )


class AsyncAutogeneratedDraftTestCasesResourceWithRawResponse:
    def __init__(self, autogenerated_draft_test_cases: AsyncAutogeneratedDraftTestCasesResource) -> None:
        self._autogenerated_draft_test_cases = autogenerated_draft_test_cases

        self.create = async_to_raw_response_wrapper(
            autogenerated_draft_test_cases.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            autogenerated_draft_test_cases.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            autogenerated_draft_test_cases.update,
        )
        self.list = async_to_raw_response_wrapper(
            autogenerated_draft_test_cases.list,
        )
        self.delete = async_to_raw_response_wrapper(
            autogenerated_draft_test_cases.delete,
        )
        self.approve = async_to_raw_response_wrapper(
            autogenerated_draft_test_cases.approve,
        )


class AutogeneratedDraftTestCasesResourceWithStreamingResponse:
    def __init__(self, autogenerated_draft_test_cases: AutogeneratedDraftTestCasesResource) -> None:
        self._autogenerated_draft_test_cases = autogenerated_draft_test_cases

        self.create = to_streamed_response_wrapper(
            autogenerated_draft_test_cases.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            autogenerated_draft_test_cases.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            autogenerated_draft_test_cases.update,
        )
        self.list = to_streamed_response_wrapper(
            autogenerated_draft_test_cases.list,
        )
        self.delete = to_streamed_response_wrapper(
            autogenerated_draft_test_cases.delete,
        )
        self.approve = to_streamed_response_wrapper(
            autogenerated_draft_test_cases.approve,
        )


class AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse:
    def __init__(self, autogenerated_draft_test_cases: AsyncAutogeneratedDraftTestCasesResource) -> None:
        self._autogenerated_draft_test_cases = autogenerated_draft_test_cases

        self.create = async_to_streamed_response_wrapper(
            autogenerated_draft_test_cases.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            autogenerated_draft_test_cases.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            autogenerated_draft_test_cases.update,
        )
        self.list = async_to_streamed_response_wrapper(
            autogenerated_draft_test_cases.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            autogenerated_draft_test_cases.delete,
        )
        self.approve = async_to_streamed_response_wrapper(
            autogenerated_draft_test_cases.approve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PaginatedArtifacts", "Item", "ItemChunksStatus"]


class ItemChunksStatus(BaseModel):
    chunks_completed: int

    chunks_failed: int

    chunks_pending: int


class Item(BaseModel):
    artifact_id: str

    artifact_name: str

    artifact_uri: str

    chunks_status: ItemChunksStatus

    source: Literal[
        "S3", "SharePoint", "LocalFile", "LocalChunks", "GoogleDrive", "AzureBlobStorage", "Confluence", "Slack"
    ]
    """An enumeration."""

    status: Literal["Pending", "Chunking", "Uploading", "Completed", "Failed", "Deleting", "Canceled", "Embedding"]
    """An enumeration."""

    artifact_uri_public: Optional[str] = None

    created_at: Optional[datetime] = None

    status_reason: Optional[str] = None

    updated_at: Optional[datetime] = None


class PaginatedArtifacts(BaseModel):
    current_page: int
    """The current page number."""

    items: List[Item]
    """The data returned for the current page."""

    items_per_page: int
    """The number of items per page."""

    total_item_count: int
    """The total number of items of the query"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["InteractionCreateParams", "Input", "Output", "OutputContext", "TraceSpan"]


class InteractionCreateParams(TypedDict, total=False):
    application_variant_id: Required[str]
    """Identifier for the application variant that performed this interaction."""

    input: Required[Input]
    """Represents the input of an interaction."""

    output: Required[Output]
    """Represents the output of an interaction."""

    start_timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Timestamp marking the start of the interaction."""

    duration_ms: int
    """Duration of the interaction in milliseconds."""

    operation_metadata: object
    """
    Optional metadata related to the operation, including custom or predefined keys.
    """

    operation_status: Literal["SUCCESS", "ERROR"]
    """Enum representing the status of an operation."""

    thread_id: str
    """
    Optional UUID identifying the conversation thread associated with the
    interaction.The interaction will be associated with the thread if the id
    represents an existing thread.If the thread with the specified id is not found,
    a new thread will be created.
    """

    trace_spans: Iterable[TraceSpan]
    """
    List of trace spans associated with the interaction.These spans provide insight
    into the individual steps taken by nodes involved in generating the output.
    """


class Input(TypedDict, total=False):
    query: Required[str]
    """The query or input text for the interaction."""


class OutputContext(TypedDict, total=False):
    text: Required[str]
    """The text of the context entry."""

    score: float
    """The score of the context entry."""


class Output(TypedDict, total=False):
    response: Required[str]
    """The response or output text of the interaction."""

    context: Iterable[OutputContext]
    """Optional context information provided with the response."""


class TraceSpan(TypedDict, total=False):
    node_id: Required[str]
    """Identifier for the node that emitted this trace span."""

    operation_type: Required[Literal["COMPLETION", "RERANKING", "RETRIEVAL", "CUSTOM"]]
    """Enum representing the type of operation performed."""

    start_timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The start time of the step."""

    duration_ms: int
    """The duration of the operation step in milliseconds."""

    operation_input: object
    """The JSON representation of the input that this step received."""

    operation_metadata: object
    """The JSON representation of the metadata insights emitted during execution.

    This can differ based on different types of operations.
    """

    operation_output: object
    """The JSON representation of the output that this step emitted."""

    operation_status: Literal["SUCCESS", "ERROR"]
    """Enum representing the status of an operation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Dashboard", "Visualization", "VisualizationMetadata"]


class VisualizationMetadata(BaseModel):
    is_variant_specific_metric: Optional[bool] = None

    label_color_map: Optional[Dict[str, str]] = None

    label_title_map: Optional[Dict[str, str]] = None


class Visualization(BaseModel):
    id: str

    metric_id: Literal[
        "total_requests",
        "total_errors",
        "total_tokens",
        "average_latency",
        "p95_latency",
        "error_rate",
        "average_faithfulness",
        "average_relevance",
        "average_users",
        "aggregated_tokens",
        "feedback",
        "engagement_faithfulness",
        "engagement_relevance",
        "execution_input_response_tokens",
        "execution_average_latency_per_variant",
        "execution_error_rate",
        "execution_latency_percentile",
        "execution_average_latency_per_node",
    ]
    """An enumeration."""

    title: str

    type: Literal["scalar", "bar", "stacked_bar", "line"]
    """An enumeration."""

    metadata: Optional[VisualizationMetadata] = None


class Dashboard(BaseModel):
    id: str

    title: str

    visualizations: List[Visualization]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Required, TypeAlias, TypedDict

from .flexible_chunk_param import FlexibleChunkParam
from .flexible_message_param import FlexibleMessageParam
from ..shared_params.chunk_extra_info_schema import ChunkExtraInfoSchema
from ..shared_params.string_extra_info_schema import StringExtraInfoSchema

__all__ = ["FlexibleTestCaseSchemaParam", "ExpectedExtraInfo"]

ExpectedExtraInfo: TypeAlias = Union[ChunkExtraInfoSchema, StringExtraInfoSchema]


class FlexibleTestCaseSchemaParam(TypedDict, total=False):
    input: Required[
        Union[
            str,
            float,
            Iterable[FlexibleChunkParam],
            Iterable[FlexibleMessageParam],
            Iterable[object],
            Dict[
                str,
                Union[
                    str, float, Iterable[FlexibleChunkParam], Iterable[FlexibleMessageParam], Iterable[object], object
                ],
            ],
            object,
        ]
    ]

    expected_extra_info: ExpectedExtraInfo

    expected_output: Union[
        str,
        float,
        Iterable[FlexibleChunkParam],
        Iterable[FlexibleMessageParam],
        Iterable[object],
        Dict[
            str,
            Union[str, float, Iterable[FlexibleChunkParam], Iterable[FlexibleMessageParam], Iterable[object], object],
        ],
        object,
    ]



### completions_example_script.py ###
# This script demonstrates how to create a completion using the Scale GP API client
# 1. How to call a specified model for a completion
# 2. How to call a specified model for a chat completion, allowing for message history

import os

from scale_gp import SGPClient
from scale_gp.types.completion_create_params import ModelParameters
from scale_gp.types.chat_completion_create_params import (
    ModelParameters as ChatModelParameters,
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage,
)

account_id = os.environ.get("SGP_ACCOUNT_ID", None)
api_key = os.environ.get("SGP_API_KEY", None)

assert (
    account_id is not None
), "You need to set the SGP_ACCOUNT_ID - you can find it at https://egp.dashboard.scale.com/admin/accounts"
assert api_key is not None, "You need to provide your API key - see https://egp.dashboard.scale.com/admin/api-key"

# Create an SGP Client
client = SGPClient(api_key=api_key)

model_parameters = ModelParameters(max_tokens=200, temperature=0.5, top_k=1, top_p=1)

# 1. call a specified model to perform a completion using a prompt and the parameters defined above
completion = client.completions.create(
    model="gpt-4",
    prompt="Why is the sky blue?",
    account_id=account_id,
    model_parameters=model_parameters,
)

print(completion.completion.text)
# Returns: The sky appears blue because of a process called Rayleigh scattering. As sunlight reaches Earth's atmosphere, it is scattered, or redirected, in every direction by the gases and particles in the air. Blue light is scattered more than other colors because it travels in shorter, smaller waves. This scattered blue light is what we see when we look up at the sky.

# 2. call a chat model to perform a completion using a message and the parameters defined above. Chat completion allows for a series of messages to be sent to the model,
# serving as a conversation between the user and the model
chat_model_parameters = ChatModelParameters(max_tokens=200, temperature=0.5, top_k=1, top_p=1)
message = MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage(role="user", content="What is the capital of Canada?")

chat_completion = client.chat_completions.create(
    model="gpt-4",
    messages=[
        message
    ],  # messages is a list of historical messages in the conversation, with roles usually alternating between user, assistant and system
    account_id=account_id,
    model_parameters=model_parameters,
    instructions="Answer the question like an elementary school teacher.",
)

print(chat_completion.chat_completion.message.content)
# Returns: The capital of Canada is Ottawa. Ottawa is a beautiful city located in the eastern part of the country, in the province of Ontario. It's where the Prime Minister lives and where many important decisions for the country are made!



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .parameter_schema_field import ParameterSchemaField

__all__ = ["ParameterSchema"]


class ParameterSchema(BaseModel):
    parameters: List[ParameterSchemaField]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import (
    DeploymentPackageJob,
    InstallDeploymentPackageResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDeploymentPackages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_install(self, client: SGPClient) -> None:
        deployment_package = client.deployment_packages.install(
            account_id="account_id",
        )
        assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

    @parametrize
    def test_method_install_with_all_params(self, client: SGPClient) -> None:
        deployment_package = client.deployment_packages.install(
            account_id="account_id",
            deployment_package_id="deployment_package_id",
            deployment_package_name="deployment_package_name",
            deployment_package_type="EGP_CONFIGURATION",
            version_constraint={
                "minimum_version": "minimum_version",
                "version_constraint_type": "SEMVER_MIN",
            },
        )
        assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

    @parametrize
    def test_raw_response_install(self, client: SGPClient) -> None:
        response = client.deployment_packages.with_raw_response.install(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment_package = response.parse()
        assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

    @parametrize
    def test_streaming_response_install(self, client: SGPClient) -> None:
        with client.deployment_packages.with_streaming_response.install(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment_package = response.parse()
            assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_install(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.deployment_packages.with_raw_response.install(
                account_id="",
            )

    @parametrize
    def test_method_install_async(self, client: SGPClient) -> None:
        deployment_package = client.deployment_packages.install_async(
            account_id="account_id",
        )
        assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

    @parametrize
    def test_method_install_async_with_all_params(self, client: SGPClient) -> None:
        deployment_package = client.deployment_packages.install_async(
            account_id="account_id",
            deployment_package_id="deployment_package_id",
            deployment_package_name="deployment_package_name",
            deployment_package_type="EGP_CONFIGURATION",
            version_constraint={
                "minimum_version": "minimum_version",
                "version_constraint_type": "SEMVER_MIN",
            },
        )
        assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

    @parametrize
    def test_raw_response_install_async(self, client: SGPClient) -> None:
        response = client.deployment_packages.with_raw_response.install_async(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment_package = response.parse()
        assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

    @parametrize
    def test_streaming_response_install_async(self, client: SGPClient) -> None:
        with client.deployment_packages.with_streaming_response.install_async(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment_package = response.parse()
            assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_install_async(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.deployment_packages.with_raw_response.install_async(
                account_id="",
            )


class TestAsyncDeploymentPackages:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_install(self, async_client: AsyncSGPClient) -> None:
        deployment_package = await async_client.deployment_packages.install(
            account_id="account_id",
        )
        assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

    @parametrize
    async def test_method_install_with_all_params(self, async_client: AsyncSGPClient) -> None:
        deployment_package = await async_client.deployment_packages.install(
            account_id="account_id",
            deployment_package_id="deployment_package_id",
            deployment_package_name="deployment_package_name",
            deployment_package_type="EGP_CONFIGURATION",
            version_constraint={
                "minimum_version": "minimum_version",
                "version_constraint_type": "SEMVER_MIN",
            },
        )
        assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

    @parametrize
    async def test_raw_response_install(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.deployment_packages.with_raw_response.install(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment_package = await response.parse()
        assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

    @parametrize
    async def test_streaming_response_install(self, async_client: AsyncSGPClient) -> None:
        async with async_client.deployment_packages.with_streaming_response.install(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment_package = await response.parse()
            assert_matches_type(InstallDeploymentPackageResponse, deployment_package, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_install(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.deployment_packages.with_raw_response.install(
                account_id="",
            )

    @parametrize
    async def test_method_install_async(self, async_client: AsyncSGPClient) -> None:
        deployment_package = await async_client.deployment_packages.install_async(
            account_id="account_id",
        )
        assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

    @parametrize
    async def test_method_install_async_with_all_params(self, async_client: AsyncSGPClient) -> None:
        deployment_package = await async_client.deployment_packages.install_async(
            account_id="account_id",
            deployment_package_id="deployment_package_id",
            deployment_package_name="deployment_package_name",
            deployment_package_type="EGP_CONFIGURATION",
            version_constraint={
                "minimum_version": "minimum_version",
                "version_constraint_type": "SEMVER_MIN",
            },
        )
        assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

    @parametrize
    async def test_raw_response_install_async(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.deployment_packages.with_raw_response.install_async(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment_package = await response.parse()
        assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

    @parametrize
    async def test_streaming_response_install_async(self, async_client: AsyncSGPClient) -> None:
        async with async_client.deployment_packages.with_streaming_response.install_async(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment_package = await response.parse()
            assert_matches_type(DeploymentPackageJob, deployment_package, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_install_async(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.deployment_packages.with_raw_response.install_async(
                account_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from ..shared_params.chunk_extra_info_schema import ChunkExtraInfoSchema
from ..shared_params.string_extra_info_schema import StringExtraInfoSchema

__all__ = ["GenerationTestCaseSchemaParam", "ExpectedExtraInfo"]

ExpectedExtraInfo: TypeAlias = Union[ChunkExtraInfoSchema, StringExtraInfoSchema]


class GenerationTestCaseSchemaParam(TypedDict, total=False):
    input: Required[str]

    expected_extra_info: ExpectedExtraInfo

    expected_output: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo
from .shared_params.chunk_extra_info_schema import ChunkExtraInfoSchema
from .shared_params.result_schema_generation import ResultSchemaGeneration
from .shared_params.string_extra_info_schema import StringExtraInfoSchema
from .evaluation_datasets.flexible_chunk_param import FlexibleChunkParam
from .evaluation_datasets.flexible_message_param import FlexibleMessageParam

__all__ = [
    "ApplicationTestCaseOutputBatchParams",
    "Item",
    "ItemOutput",
    "ItemOutputResultSchemaFlexible",
    "ItemOutputResultSchemaFlexibleGenerationExtraInfo",
    "ItemTraceSpan",
]


class ApplicationTestCaseOutputBatchParams(TypedDict, total=False):
    items: Required[Iterable[Item]]


ItemOutputResultSchemaFlexibleGenerationExtraInfo: TypeAlias = Union[ChunkExtraInfoSchema, StringExtraInfoSchema]


class ItemOutputResultSchemaFlexible(TypedDict, total=False):
    generation_output: Required[
        Union[
            str,
            float,
            Iterable[FlexibleChunkParam],
            Iterable[FlexibleMessageParam],
            Iterable[object],
            Dict[
                str,
                Union[
                    str, float, Iterable[FlexibleChunkParam], Iterable[FlexibleMessageParam], Iterable[object], object
                ],
            ],
            object,
        ]
    ]

    generation_extra_info: ItemOutputResultSchemaFlexibleGenerationExtraInfo


ItemOutput: TypeAlias = Union[ResultSchemaGeneration, ItemOutputResultSchemaFlexible]


class ItemTraceSpan(TypedDict, total=False):
    node_id: Required[str]
    """Identifier for the node that emitted this trace span."""

    operation_input: Required[
        Dict[
            str,
            Union[str, float, Iterable[FlexibleChunkParam], Iterable[FlexibleMessageParam], Iterable[object], object],
        ]
    ]
    """The JSON representation of the input that this step received."""

    operation_output: Required[
        Dict[
            str,
            Union[str, float, Iterable[FlexibleChunkParam], Iterable[FlexibleMessageParam], Iterable[object], object],
        ]
    ]
    """The JSON representation of the output that this step emitted."""

    start_timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The start time of the step."""

    duration_ms: int
    """The duration of the operation step in milliseconds."""

    operation_expected: Dict[
        str, Union[str, float, Iterable[FlexibleChunkParam], Iterable[FlexibleMessageParam], Iterable[object], object]
    ]
    """The JSON representation of the expected output for this step"""

    operation_metadata: object
    """The JSON representation of the metadata insights emitted during execution.

    This can differ based on different types of operations.
    """

    operation_status: Literal["SUCCESS", "ERROR"]
    """Enum representing the status of an operation."""

    operation_type: Literal["COMPLETION", "RERANKING", "RETRIEVAL", "CUSTOM"]
    """Enum representing the type of operation performed."""


class Item(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    application_variant_id: Required[str]

    evaluation_dataset_version_num: Required[int]

    output: Required[ItemOutput]

    test_case_id: Required[str]

    application_interaction_id: str

    metrics: Dict[str, float]

    trace_spans: Iterable[ItemTraceSpan]
    """List of trace spans associated with the application's execution.

    These spans provide insight into the individual steps taken by nodes involved in
    generating the output.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EvaluationConfigCreateParams"]


class EvaluationConfigCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    evaluation_type: Required[Literal["studio", "llm_auto", "human", "llm_benchmark"]]
    """An enumeration."""

    question_set_id: Required[str]

    studio_project_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .s3_data_source_config import S3DataSourceConfig
from .slack_data_source_config import SlackDataSourceConfig
from .confluence_data_source_config import ConfluenceDataSourceConfig
from .share_point_data_source_config import SharePointDataSourceConfig
from .google_drive_data_source_config import GoogleDriveDataSourceConfig
from .azure_blob_storage_data_source_config import AzureBlobStorageDataSourceConfig

__all__ = ["KnowledgeBaseDataSource", "DataSourceConfig"]

DataSourceConfig: TypeAlias = Annotated[
    Union[
        S3DataSourceConfig,
        SharePointDataSourceConfig,
        GoogleDriveDataSourceConfig,
        AzureBlobStorageDataSourceConfig,
        ConfluenceDataSourceConfig,
        SlackDataSourceConfig,
    ],
    PropertyInfo(discriminator="source"),
]


class KnowledgeBaseDataSource(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    data_source_config: DataSourceConfig

    name: str

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    description: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import ChatThread
from scale_gp.pagination import SyncTopLevelArray, AsyncTopLevelArray

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChatThreads:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        chat_thread = client.applications.chat_threads.list(
            "application_variant_id",
        )
        assert_matches_type(SyncTopLevelArray[ChatThread], chat_thread, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.applications.chat_threads.with_raw_response.list(
            "application_variant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_thread = response.parse()
        assert_matches_type(SyncTopLevelArray[ChatThread], chat_thread, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.applications.chat_threads.with_streaming_response.list(
            "application_variant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_thread = response.parse()
            assert_matches_type(SyncTopLevelArray[ChatThread], chat_thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            client.applications.chat_threads.with_raw_response.list(
                "",
            )


class TestAsyncChatThreads:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        chat_thread = await async_client.applications.chat_threads.list(
            "application_variant_id",
        )
        assert_matches_type(AsyncTopLevelArray[ChatThread], chat_thread, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.applications.chat_threads.with_raw_response.list(
            "application_variant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_thread = await response.parse()
        assert_matches_type(AsyncTopLevelArray[ChatThread], chat_thread, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.applications.chat_threads.with_streaming_response.list(
            "application_variant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_thread = await response.parse()
            assert_matches_type(AsyncTopLevelArray[ChatThread], chat_thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            await async_client.applications.chat_threads.with_raw_response.list(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .evaluation_datasets.approve_auto_generated_draft_test_case_response import (
    ApproveAutoGeneratedDraftTestCaseResponse,
)

__all__ = ["PublishEvaluationDatasetDraftResponse"]


class PublishEvaluationDatasetDraftResponse(BaseModel):
    autogenerated_draft_test_cases: List[ApproveAutoGeneratedDraftTestCaseResponse]
    """List of responses for each of the input draft test cases."""

    success: bool
    """Whether or not the evaluation dataset was successfully published"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ThemeCreateParams", "ThemeVars"]


class ThemeCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    logo_blob: Required[str]

    theme_vars: Required[ThemeVars]

    title: Required[str]


class ThemeVars(TypedDict, total=False):
    accent_primary: Annotated[str, PropertyInfo(alias="accentPrimary")]

    accent_secondary: Annotated[str, PropertyInfo(alias="accentSecondary")]

    background: str

    foreground: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, Iterable, cast
from typing_extensions import Literal

import httpx

from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPageResponse, AsyncPageResponse
from ...._base_client import AsyncPaginator, make_request_options
from ....types.evaluations import (
    test_case_result_list_params,
    test_case_result_batch_params,
    test_case_result_create_params,
    test_case_result_update_params,
    test_case_result_retrieve_params,
)
from ....types.evaluations.test_case_result import TestCaseResult
from ....types.evaluations.test_case_result_with_views import TestCaseResultWithViews
from ....types.evaluations.test_case_result_batch_response import TestCaseResultBatchResponse

__all__ = ["TestCaseResultsResource", "AsyncTestCaseResultsResource"]


class TestCaseResultsResource(SyncAPIResource):
    __test__ = False

    @cached_property
    def history(self) -> HistoryResource:
        return HistoryResource(self._client)

    @cached_property
    def with_raw_response(self) -> TestCaseResultsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return TestCaseResultsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TestCaseResultsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return TestCaseResultsResourceWithStreamingResponse(self)

    def create(
        self,
        evaluation_id: str,
        *,
        application_spec_id: str,
        evaluation_dataset_version_num: str,
        test_case_evaluation_data: object,
        test_case_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        annotated_by_user_id: str | NotGiven = NOT_GIVEN,
        audit_comment: str | NotGiven = NOT_GIVEN,
        audit_required: bool | NotGiven = NOT_GIVEN,
        audit_status: Literal["UNAUDITED", "FIXED", "APPROVED"] | NotGiven = NOT_GIVEN,
        result: object | NotGiven = NOT_GIVEN,
        time_spent_labeling_s: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResult:
        """
        ### Description

        Creates a test case result

        ### Details

        This API can be used to create a test case result. To use this API, review the
        request schema and pass in all fields that are required to create a test case
        result.

        Args:
          account_id: The ID of the account that owns the given entity.

          annotated_by_user_id: The user who annotated the task.

          audit_status: An enumeration.

          time_spent_labeling_s: The time spent labeling in seconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return cast(
            TestCaseResult,
            self._post(
                f"/v4/evaluations/{evaluation_id}/test-case-results",
                body=maybe_transform(
                    {
                        "application_spec_id": application_spec_id,
                        "evaluation_dataset_version_num": evaluation_dataset_version_num,
                        "test_case_evaluation_data": test_case_evaluation_data,
                        "test_case_id": test_case_id,
                        "account_id": account_id,
                        "annotated_by_user_id": annotated_by_user_id,
                        "audit_comment": audit_comment,
                        "audit_required": audit_required,
                        "audit_status": audit_status,
                        "result": result,
                        "time_spent_labeling_s": time_spent_labeling_s,
                    },
                    test_case_result_create_params.TestCaseResultCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        test_case_result_id: str,
        *,
        evaluation_id: str,
        view: List[Literal["AnnotationResults", "Metrics", "Task", "TestCaseVersion"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResultWithViews:
        """
        ### Description

        Gets the details of a test case result

        ### Details

        This API can be used to get information about a single test case result by ID.
        To use this API, pass in the `id` that was returned from your Create Test Case
        Result API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not test_case_result_id:
            raise ValueError(
                f"Expected a non-empty value for `test_case_result_id` but received {test_case_result_id!r}"
            )
        return cast(
            TestCaseResultWithViews,
            self._get(
                f"/v4/evaluations/{evaluation_id}/test-case-results/{test_case_result_id}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform(
                        {"view": view}, test_case_result_retrieve_params.TestCaseResultRetrieveParams
                    ),
                ),
                cast_to=cast(
                    Any, TestCaseResultWithViews
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def update(
        self,
        test_case_result_id: str,
        *,
        evaluation_id: str,
        annotated_by_user_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: str | NotGiven = NOT_GIVEN,
        audit_comment: str | NotGiven = NOT_GIVEN,
        audit_required: bool | NotGiven = NOT_GIVEN,
        audit_status: Literal["UNAUDITED", "FIXED", "APPROVED"] | NotGiven = NOT_GIVEN,
        evaluation_dataset_version_num: str | NotGiven = NOT_GIVEN,
        result: object | NotGiven = NOT_GIVEN,
        test_case_evaluation_data: object | NotGiven = NOT_GIVEN,
        test_case_id: str | NotGiven = NOT_GIVEN,
        time_spent_labeling_s: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResult:
        """
        ### Description

        Updates a test case result

        ### Details

        This API can be used to update the test case result that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Test Case Result API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          annotated_by_user_id: The user who annotated the task.

          audit_status: An enumeration.

          time_spent_labeling_s: The time spent labeling in seconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not test_case_result_id:
            raise ValueError(
                f"Expected a non-empty value for `test_case_result_id` but received {test_case_result_id!r}"
            )
        return cast(
            TestCaseResult,
            self._patch(
                f"/v4/evaluations/{evaluation_id}/test-case-results/{test_case_result_id}",
                body=maybe_transform(
                    {
                        "annotated_by_user_id": annotated_by_user_id,
                        "application_spec_id": application_spec_id,
                        "audit_comment": audit_comment,
                        "audit_required": audit_required,
                        "audit_status": audit_status,
                        "evaluation_dataset_version_num": evaluation_dataset_version_num,
                        "result": result,
                        "test_case_evaluation_data": test_case_evaluation_data,
                        "test_case_id": test_case_id,
                        "time_spent_labeling_s": time_spent_labeling_s,
                    },
                    test_case_result_update_params.TestCaseResultUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        evaluation_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["AnnotationResults", "Metrics", "Task", "TestCaseVersion"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[TestCaseResultWithViews]:
        """
        ### Description

        Lists all test case results accessible to the user.

        ### Details

        This API can be used to list test case results. If a user has access to multiple
        accounts, all test case results from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._get_api_list(
            f"/v4/evaluations/{evaluation_id}/test-case-results",
            page=SyncPageResponse[TestCaseResultWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    test_case_result_list_params.TestCaseResultListParams,
                ),
            ),
            model=cast(Any, TestCaseResultWithViews),  # Union types cannot be passed in as arguments in the type system
        )

    def batch(
        self,
        evaluation_id: str,
        *,
        items: Iterable[test_case_result_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResultBatchResponse:
        """
        ### Description

        Creates a batch of test case results

        ### Details

        This API can be used to create multiple test case results so users do not have
        to the incur the cost of repeated network calls. To use this API, pass in a list
        of test case results in the request body.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._post(
            f"/v4/evaluations/{evaluation_id}/test-case-results/batch",
            body=maybe_transform(items, Iterable[test_case_result_batch_params.Item]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestCaseResultBatchResponse,
        )


class AsyncTestCaseResultsResource(AsyncAPIResource):
    @cached_property
    def history(self) -> AsyncHistoryResource:
        return AsyncHistoryResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTestCaseResultsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTestCaseResultsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTestCaseResultsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncTestCaseResultsResourceWithStreamingResponse(self)

    async def create(
        self,
        evaluation_id: str,
        *,
        application_spec_id: str,
        evaluation_dataset_version_num: str,
        test_case_evaluation_data: object,
        test_case_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        annotated_by_user_id: str | NotGiven = NOT_GIVEN,
        audit_comment: str | NotGiven = NOT_GIVEN,
        audit_required: bool | NotGiven = NOT_GIVEN,
        audit_status: Literal["UNAUDITED", "FIXED", "APPROVED"] | NotGiven = NOT_GIVEN,
        result: object | NotGiven = NOT_GIVEN,
        time_spent_labeling_s: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResult:
        """
        ### Description

        Creates a test case result

        ### Details

        This API can be used to create a test case result. To use this API, review the
        request schema and pass in all fields that are required to create a test case
        result.

        Args:
          account_id: The ID of the account that owns the given entity.

          annotated_by_user_id: The user who annotated the task.

          audit_status: An enumeration.

          time_spent_labeling_s: The time spent labeling in seconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return cast(
            TestCaseResult,
            await self._post(
                f"/v4/evaluations/{evaluation_id}/test-case-results",
                body=await async_maybe_transform(
                    {
                        "application_spec_id": application_spec_id,
                        "evaluation_dataset_version_num": evaluation_dataset_version_num,
                        "test_case_evaluation_data": test_case_evaluation_data,
                        "test_case_id": test_case_id,
                        "account_id": account_id,
                        "annotated_by_user_id": annotated_by_user_id,
                        "audit_comment": audit_comment,
                        "audit_required": audit_required,
                        "audit_status": audit_status,
                        "result": result,
                        "time_spent_labeling_s": time_spent_labeling_s,
                    },
                    test_case_result_create_params.TestCaseResultCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        test_case_result_id: str,
        *,
        evaluation_id: str,
        view: List[Literal["AnnotationResults", "Metrics", "Task", "TestCaseVersion"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResultWithViews:
        """
        ### Description

        Gets the details of a test case result

        ### Details

        This API can be used to get information about a single test case result by ID.
        To use this API, pass in the `id` that was returned from your Create Test Case
        Result API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not test_case_result_id:
            raise ValueError(
                f"Expected a non-empty value for `test_case_result_id` but received {test_case_result_id!r}"
            )
        return cast(
            TestCaseResultWithViews,
            await self._get(
                f"/v4/evaluations/{evaluation_id}/test-case-results/{test_case_result_id}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform(
                        {"view": view}, test_case_result_retrieve_params.TestCaseResultRetrieveParams
                    ),
                ),
                cast_to=cast(
                    Any, TestCaseResultWithViews
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def update(
        self,
        test_case_result_id: str,
        *,
        evaluation_id: str,
        annotated_by_user_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: str | NotGiven = NOT_GIVEN,
        audit_comment: str | NotGiven = NOT_GIVEN,
        audit_required: bool | NotGiven = NOT_GIVEN,
        audit_status: Literal["UNAUDITED", "FIXED", "APPROVED"] | NotGiven = NOT_GIVEN,
        evaluation_dataset_version_num: str | NotGiven = NOT_GIVEN,
        result: object | NotGiven = NOT_GIVEN,
        test_case_evaluation_data: object | NotGiven = NOT_GIVEN,
        test_case_id: str | NotGiven = NOT_GIVEN,
        time_spent_labeling_s: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResult:
        """
        ### Description

        Updates a test case result

        ### Details

        This API can be used to update the test case result that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Test Case Result API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          annotated_by_user_id: The user who annotated the task.

          audit_status: An enumeration.

          time_spent_labeling_s: The time spent labeling in seconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not test_case_result_id:
            raise ValueError(
                f"Expected a non-empty value for `test_case_result_id` but received {test_case_result_id!r}"
            )
        return cast(
            TestCaseResult,
            await self._patch(
                f"/v4/evaluations/{evaluation_id}/test-case-results/{test_case_result_id}",
                body=await async_maybe_transform(
                    {
                        "annotated_by_user_id": annotated_by_user_id,
                        "application_spec_id": application_spec_id,
                        "audit_comment": audit_comment,
                        "audit_required": audit_required,
                        "audit_status": audit_status,
                        "evaluation_dataset_version_num": evaluation_dataset_version_num,
                        "result": result,
                        "test_case_evaluation_data": test_case_evaluation_data,
                        "test_case_id": test_case_id,
                        "time_spent_labeling_s": time_spent_labeling_s,
                    },
                    test_case_result_update_params.TestCaseResultUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCaseResult),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        evaluation_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["AnnotationResults", "Metrics", "Task", "TestCaseVersion"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[TestCaseResultWithViews, AsyncPageResponse[TestCaseResultWithViews]]:
        """
        ### Description

        Lists all test case results accessible to the user.

        ### Details

        This API can be used to list test case results. If a user has access to multiple
        accounts, all test case results from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._get_api_list(
            f"/v4/evaluations/{evaluation_id}/test-case-results",
            page=AsyncPageResponse[TestCaseResultWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    test_case_result_list_params.TestCaseResultListParams,
                ),
            ),
            model=cast(Any, TestCaseResultWithViews),  # Union types cannot be passed in as arguments in the type system
        )

    async def batch(
        self,
        evaluation_id: str,
        *,
        items: Iterable[test_case_result_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseResultBatchResponse:
        """
        ### Description

        Creates a batch of test case results

        ### Details

        This API can be used to create multiple test case results so users do not have
        to the incur the cost of repeated network calls. To use this API, pass in a list
        of test case results in the request body.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return await self._post(
            f"/v4/evaluations/{evaluation_id}/test-case-results/batch",
            body=await async_maybe_transform(items, Iterable[test_case_result_batch_params.Item]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestCaseResultBatchResponse,
        )


class TestCaseResultsResourceWithRawResponse:
    __test__ = False

    def __init__(self, test_case_results: TestCaseResultsResource) -> None:
        self._test_case_results = test_case_results

        self.create = to_raw_response_wrapper(
            test_case_results.create,
        )
        self.retrieve = to_raw_response_wrapper(
            test_case_results.retrieve,
        )
        self.update = to_raw_response_wrapper(
            test_case_results.update,
        )
        self.list = to_raw_response_wrapper(
            test_case_results.list,
        )
        self.batch = to_raw_response_wrapper(
            test_case_results.batch,
        )

    @cached_property
    def history(self) -> HistoryResourceWithRawResponse:
        return HistoryResourceWithRawResponse(self._test_case_results.history)


class AsyncTestCaseResultsResourceWithRawResponse:
    def __init__(self, test_case_results: AsyncTestCaseResultsResource) -> None:
        self._test_case_results = test_case_results

        self.create = async_to_raw_response_wrapper(
            test_case_results.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            test_case_results.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            test_case_results.update,
        )
        self.list = async_to_raw_response_wrapper(
            test_case_results.list,
        )
        self.batch = async_to_raw_response_wrapper(
            test_case_results.batch,
        )

    @cached_property
    def history(self) -> AsyncHistoryResourceWithRawResponse:
        return AsyncHistoryResourceWithRawResponse(self._test_case_results.history)


class TestCaseResultsResourceWithStreamingResponse:
    __test__ = False

    def __init__(self, test_case_results: TestCaseResultsResource) -> None:
        self._test_case_results = test_case_results

        self.create = to_streamed_response_wrapper(
            test_case_results.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            test_case_results.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            test_case_results.update,
        )
        self.list = to_streamed_response_wrapper(
            test_case_results.list,
        )
        self.batch = to_streamed_response_wrapper(
            test_case_results.batch,
        )

    @cached_property
    def history(self) -> HistoryResourceWithStreamingResponse:
        return HistoryResourceWithStreamingResponse(self._test_case_results.history)


class AsyncTestCaseResultsResourceWithStreamingResponse:
    def __init__(self, test_case_results: AsyncTestCaseResultsResource) -> None:
        self._test_case_results = test_case_results

        self.create = async_to_streamed_response_wrapper(
            test_case_results.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            test_case_results.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            test_case_results.update,
        )
        self.list = async_to_streamed_response_wrapper(
            test_case_results.list,
        )
        self.batch = async_to_streamed_response_wrapper(
            test_case_results.batch,
        )

    @cached_property
    def history(self) -> AsyncHistoryResourceWithStreamingResponse:
        return AsyncHistoryResourceWithStreamingResponse(self._test_case_results.history)
